#include "device.h"
#include "utils/array.h"
#include "usb/usb.h"
#include <dlfcn.h>
#include <string.h>

struct DEVICE
{
    // Device driver
    void* driver;

    // Physical device handle
    void* handle;
};

array_t* device_array;

void device_init()
{
    device_array = array_alloc();
    usb_init();
}

array_t* device_get_array()
{
    return device_array;
}

device_t* device_register(void* driver, void* handle)
{
    device_t* tr = malloc(sizeof(device_t));
    tr->driver = driver;
    tr->handle = handle;

    array_add(device_array, tr);

    return tr;
}

char* device_get_image(device_t* device)
{
    char* (*getimage_fn)(void*) = dlsym(device->driver, "driver_get_image");
    char* image = getimage_fn(device->handle);
    size_t image_len = strlen(image);
    
    // TODO : Change PATH so that 'build' becomes executable directory
    char* path = "build/drivers/assets/";
    size_t path_len = strlen(path);

    char* tr = malloc(image_len + path_len + 1);
    memcpy(tr, path, path_len);
    memcpy(tr + path_len, image, image_len);
    *(tr + path_len + image_len) = 0;

    return tr;
}

char* device_get_name(device_t* device)
{
    char* (*getname_fn)(void*) = dlsym(device->driver, "driver_get_name");
    return getname_fn(device->handle);
}
